# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='messages.proto',
  package='messages',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0emessages.proto\x12\x08messages\"\xa4\x01\n\x08Response\x12(\n\x06status\x18\x01 \x01(\x0e\x32\x18.messages.ResponseStatus\x12\x34\n\x0eserviceMessage\x18\x02 \x01(\x0b\x32\x1c.messages.FromServiceMessage\x12\x38\n\x10\x63onnectorMessage\x18\x03 \x01(\x0b\x32\x1e.messages.FromConnectorMessage\"\xa3\x05\n\x14\x46romConnectorMessage\x12$\n\x08response\x18\x01 \x01(\x0b\x32\x12.messages.Response\x12:\n\x13gatewayTelemetryMsg\x18\x02 \x01(\x0b\x32\x1d.messages.GatewayTelemetryMsg\x12<\n\x14gatewayAttributesMsg\x18\x03 \x01(\x0b\x32\x1e.messages.GatewayAttributesMsg\x12\x32\n\x0fgatewayClaimMsg\x18\x04 \x01(\x0b\x32\x19.messages.GatewayClaimMsg\x12<\n\x14registerConnectorMsg\x18\x05 \x01(\x0b\x32\x1e.messages.RegisterConnectorMsg\x12@\n\x16unregisterConnectorMsg\x18\x06 \x01(\x0b\x32 .messages.UnregisterConnectorMsg\x12(\n\nconnectMsg\x18\x07 \x01(\x0b\x32\x14.messages.ConnectMsg\x12.\n\rdisconnectMsg\x18\x08 \x01(\x0b\x32\x17.messages.DisconnectMsg\x12>\n\x15gatewayRpcResponseMsg\x18\t \x01(\x0b\x32\x1f.messages.GatewayRpcResponseMsg\x12I\n\x1agatewayAttributeRequestMsg\x18\n \x01(\x0b\x32%.messages.GatewayAttributesRequestMsg\x12R\n\x1f\x63onnectorGetConnectedDevicesMsg\x18\x0b \x01(\x0b\x32).messages.ConnectorGetConnectedDevicesMsg\"\x9e\x04\n\x12\x46romServiceMessage\x12$\n\x08response\x18\x01 \x01(\x0b\x32\x12.messages.Response\x12\x46\n\x19\x63onnectorConfigurationMsg\x18\x02 \x01(\x0b\x32#.messages.ConnectorConfigurationMsg\x12^\n%gatewayAttributeUpdateNotificationMsg\x18\x03 \x01(\x0b\x32/.messages.GatewayAttributeUpdateNotificationMsg\x12J\n\x1bgatewayAttributeResponseMsg\x18\x04 \x01(\x0b\x32%.messages.GatewayAttributeResponseMsg\x12H\n\x1agatewayDeviceRpcRequestMsg\x18\x05 \x01(\x0b\x32$.messages.GatewayDeviceRpcRequestMsg\x12@\n\x16unregisterConnectorMsg\x18\x06 \x01(\x0b\x32 .messages.UnregisterConnectorMsg\x12\x62\n\'connectorGetConnectedDevicesResponseMsg\x18\x07 \x01(\x0b\x32\x31.messages.ConnectorGetConnectedDevicesResponseMsg\",\n\x14RegisterConnectorMsg\x12\x14\n\x0c\x63onnectorKey\x18\x01 \x01(\t\".\n\x16UnregisterConnectorMsg\x12\x14\n\x0c\x63onnectorKey\x18\x01 \x01(\t\"I\n\x19\x43onnectorConfigurationMsg\x12\x15\n\rconnectorName\x18\x01 \x01(\t\x12\x15\n\rconfiguration\x18\x02 \x01(\t\"7\n\x1f\x43onnectorGetConnectedDevicesMsg\x12\x14\n\x0c\x63onnectorKey\x18\x01 \x01(\t\"b\n\'ConnectorGetConnectedDevicesResponseMsg\x12\x37\n\x10\x63onnectorDevices\x18\x01 \x03(\x0b\x32\x1d.messages.ConnectorDeviceInfo\"=\n\x13\x43onnectorDeviceInfo\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x12\n\ndeviceType\x18\x02 \x01(\t\"\x96\x01\n\rKeyValueProto\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x04type\x18\x02 \x01(\x0e\x32\x16.messages.KeyValueType\x12\x0e\n\x06\x62ool_v\x18\x03 \x01(\x08\x12\x0e\n\x06long_v\x18\x04 \x01(\x03\x12\x10\n\x08\x64ouble_v\x18\x05 \x01(\x01\x12\x10\n\x08string_v\x18\x06 \x01(\t\x12\x0e\n\x06json_v\x18\x07 \x01(\t\"<\n\tTsKvProto\x12\n\n\x02ts\x18\x01 \x01(\x03\x12#\n\x02kv\x18\x02 \x01(\x0b\x32\x17.messages.KeyValueProto\"@\n\rTsKvListProto\x12\n\n\x02ts\x18\x01 \x01(\x03\x12#\n\x02kv\x18\x02 \x03(\x0b\x32\x17.messages.KeyValueProto\"=\n\x10PostTelemetryMsg\x12)\n\x08tsKvList\x18\x01 \x03(\x0b\x32\x17.messages.TsKvListProto\"7\n\x10PostAttributeMsg\x12#\n\x02kv\x18\x01 \x03(\x0b\x32\x17.messages.KeyValueProto\"c\n\x1e\x41ttributeUpdateNotificationMsg\x12*\n\rsharedUpdated\x18\x01 \x03(\x0b\x32\x13.messages.TsKvProto\x12\x15\n\rsharedDeleted\x18\x02 \x03(\t\"N\n\x15ToDeviceRpcRequestMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x12\n\nmethodName\x18\x02 \x01(\t\x12\x0e\n\x06params\x18\x03 \x01(\t\"K\n\x16ToDeviceRpcResponseMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x0f\n\x07payload\x18\x02 \x01(\t\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"N\n\x15ToServerRpcRequestMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x12\n\nmethodName\x18\x02 \x01(\t\x12\x0e\n\x06params\x18\x03 \x01(\t\"K\n\x16ToServerRpcResponseMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x0f\n\x07payload\x18\x02 \x01(\t\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"4\n\x0b\x43laimDevice\x12\x11\n\tsecretKey\x18\x01 \x01(\t\x12\x12\n\ndurationMs\x18\x02 \x01(\x03\";\n\x11\x41ttributesRequest\x12\x12\n\nclientKeys\x18\x01 \x01(\t\x12\x12\n\nsharedKeys\x18\x02 \x01(\t\",\n\nRpcRequest\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x01(\t\"#\n\rDisconnectMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\"4\n\nConnectMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x12\n\ndeviceType\x18\x02 \x01(\t\"K\n\x0cTelemetryMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\'\n\x03msg\x18\x03 \x01(\x0b\x32\x1a.messages.PostTelemetryMsg\"L\n\rAttributesMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\'\n\x03msg\x18\x02 \x01(\x0b\x32\x1a.messages.PostAttributeMsg\"Q\n\x0e\x43laimDeviceMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12+\n\x0c\x63laimRequest\x18\x02 \x01(\x0b\x32\x15.messages.ClaimDevice\":\n\x13GatewayTelemetryMsg\x12#\n\x03msg\x18\x01 \x03(\x0b\x32\x16.messages.TelemetryMsg\"8\n\x0fGatewayClaimMsg\x12%\n\x03msg\x18\x01 \x03(\x0b\x32\x18.messages.ClaimDeviceMsg\"<\n\x14GatewayAttributesMsg\x12$\n\x03msg\x18\x01 \x03(\x0b\x32\x17.messages.AttributesMsg\"E\n\x15GatewayRpcResponseMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\t\"n\n\x1bGatewayAttributeResponseMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12;\n\x0bresponseMsg\x18\x02 \x01(\x0b\x32&.messages.GatewayAttributesResponseMsg\"~\n%GatewayAttributeUpdateNotificationMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x41\n\x0fnotificationMsg\x18\x02 \x01(\x0b\x32(.messages.AttributeUpdateNotificationMsg\"h\n\x1aGatewayDeviceRpcRequestMsg\x12\x12\n\ndeviceName\x18\x01 \x01(\t\x12\x36\n\rrpcRequestMsg\x18\x02 \x01(\x0b\x32\x1f.messages.ToDeviceRpcRequestMsg\"[\n\x1bGatewayAttributesRequestMsg\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x12\n\ndeviceName\x18\x02 \x01(\t\x12\x0e\n\x06\x63lient\x18\x03 \x01(\x08\x12\x0c\n\x04keys\x18\x04 \x03(\t\"\xac\x01\n\x1cGatewayAttributesResponseMsg\x12\x11\n\trequestId\x18\x01 \x01(\x05\x12\x34\n\x13\x63lientAttributeList\x18\x02 \x03(\x0b\x32\x17.messages.KeyValueProto\x12\x34\n\x13sharedAttributeList\x18\x03 \x03(\x0b\x32\x17.messages.KeyValueProto\x12\r\n\x05\x65rror\x18\x05 \x01(\t*F\n\x0eResponseStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\r\n\tNOT_FOUND\x10\x02\x12\x0b\n\x07\x46\x41ILURE\x10\x03*Q\n\x0cKeyValueType\x12\r\n\tBOOLEAN_V\x10\x00\x12\n\n\x06LONG_V\x10\x01\x12\x0c\n\x08\x44OUBLE_V\x10\x02\x12\x0c\n\x08STRING_V\x10\x03\x12\n\n\x06JSON_V\x10\x04\x32_\n\x15TBGatewayProtoService\x12\x46\n\x06stream\x12\x1e.messages.FromConnectorMessage\x1a\x1c.messages.FromServiceMessageb\x06proto3'
)

_RESPONSESTATUS = _descriptor.EnumDescriptor(
  name='ResponseStatus',
  full_name='messages.ResponseStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NOT_FOUND', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FAILURE', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3978,
  serialized_end=4048,
)
_sym_db.RegisterEnumDescriptor(_RESPONSESTATUS)

ResponseStatus = enum_type_wrapper.EnumTypeWrapper(_RESPONSESTATUS)
_KEYVALUETYPE = _descriptor.EnumDescriptor(
  name='KeyValueType',
  full_name='messages.KeyValueType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BOOLEAN_V', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LONG_V', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE_V', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRING_V', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='JSON_V', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4050,
  serialized_end=4131,
)
_sym_db.RegisterEnumDescriptor(_KEYVALUETYPE)

KeyValueType = enum_type_wrapper.EnumTypeWrapper(_KEYVALUETYPE)
UNKNOWN = 0
SUCCESS = 1
NOT_FOUND = 2
FAILURE = 3
BOOLEAN_V = 0
LONG_V = 1
DOUBLE_V = 2
STRING_V = 3
JSON_V = 4



_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='messages.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='messages.Response.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serviceMessage', full_name='messages.Response.serviceMessage', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorMessage', full_name='messages.Response.connectorMessage', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=29,
  serialized_end=193,
)


_FROMCONNECTORMESSAGE = _descriptor.Descriptor(
  name='FromConnectorMessage',
  full_name='messages.FromConnectorMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='response', full_name='messages.FromConnectorMessage.response', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayTelemetryMsg', full_name='messages.FromConnectorMessage.gatewayTelemetryMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributesMsg', full_name='messages.FromConnectorMessage.gatewayAttributesMsg', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayClaimMsg', full_name='messages.FromConnectorMessage.gatewayClaimMsg', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='registerConnectorMsg', full_name='messages.FromConnectorMessage.registerConnectorMsg', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unregisterConnectorMsg', full_name='messages.FromConnectorMessage.unregisterConnectorMsg', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectMsg', full_name='messages.FromConnectorMessage.connectMsg', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='disconnectMsg', full_name='messages.FromConnectorMessage.disconnectMsg', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayRpcResponseMsg', full_name='messages.FromConnectorMessage.gatewayRpcResponseMsg', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributeRequestMsg', full_name='messages.FromConnectorMessage.gatewayAttributeRequestMsg', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorGetConnectedDevicesMsg', full_name='messages.FromConnectorMessage.connectorGetConnectedDevicesMsg', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=196,
  serialized_end=871,
)


_FROMSERVICEMESSAGE = _descriptor.Descriptor(
  name='FromServiceMessage',
  full_name='messages.FromServiceMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='response', full_name='messages.FromServiceMessage.response', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorConfigurationMsg', full_name='messages.FromServiceMessage.connectorConfigurationMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributeUpdateNotificationMsg', full_name='messages.FromServiceMessage.gatewayAttributeUpdateNotificationMsg', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayAttributeResponseMsg', full_name='messages.FromServiceMessage.gatewayAttributeResponseMsg', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gatewayDeviceRpcRequestMsg', full_name='messages.FromServiceMessage.gatewayDeviceRpcRequestMsg', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unregisterConnectorMsg', full_name='messages.FromServiceMessage.unregisterConnectorMsg', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connectorGetConnectedDevicesResponseMsg', full_name='messages.FromServiceMessage.connectorGetConnectedDevicesResponseMsg', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=874,
  serialized_end=1416,
)


_REGISTERCONNECTORMSG = _descriptor.Descriptor(
  name='RegisterConnectorMsg',
  full_name='messages.RegisterConnectorMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorKey', full_name='messages.RegisterConnectorMsg.connectorKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1418,
  serialized_end=1462,
)


_UNREGISTERCONNECTORMSG = _descriptor.Descriptor(
  name='UnregisterConnectorMsg',
  full_name='messages.UnregisterConnectorMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorKey', full_name='messages.UnregisterConnectorMsg.connectorKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1464,
  serialized_end=1510,
)


_CONNECTORCONFIGURATIONMSG = _descriptor.Descriptor(
  name='ConnectorConfigurationMsg',
  full_name='messages.ConnectorConfigurationMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorName', full_name='messages.ConnectorConfigurationMsg.connectorName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='configuration', full_name='messages.ConnectorConfigurationMsg.configuration', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1512,
  serialized_end=1585,
)


_CONNECTORGETCONNECTEDDEVICESMSG = _descriptor.Descriptor(
  name='ConnectorGetConnectedDevicesMsg',
  full_name='messages.ConnectorGetConnectedDevicesMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorKey', full_name='messages.ConnectorGetConnectedDevicesMsg.connectorKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1587,
  serialized_end=1642,
)


_CONNECTORGETCONNECTEDDEVICESRESPONSEMSG = _descriptor.Descriptor(
  name='ConnectorGetConnectedDevicesResponseMsg',
  full_name='messages.ConnectorGetConnectedDevicesResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connectorDevices', full_name='messages.ConnectorGetConnectedDevicesResponseMsg.connectorDevices', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1644,
  serialized_end=1742,
)


_CONNECTORDEVICEINFO = _descriptor.Descriptor(
  name='ConnectorDeviceInfo',
  full_name='messages.ConnectorDeviceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.ConnectorDeviceInfo.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceType', full_name='messages.ConnectorDeviceInfo.deviceType', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1744,
  serialized_end=1805,
)


_KEYVALUEPROTO = _descriptor.Descriptor(
  name='KeyValueProto',
  full_name='messages.KeyValueProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='messages.KeyValueProto.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='messages.KeyValueProto.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bool_v', full_name='messages.KeyValueProto.bool_v', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='long_v', full_name='messages.KeyValueProto.long_v', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double_v', full_name='messages.KeyValueProto.double_v', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='string_v', full_name='messages.KeyValueProto.string_v', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='json_v', full_name='messages.KeyValueProto.json_v', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1808,
  serialized_end=1958,
)


_TSKVPROTO = _descriptor.Descriptor(
  name='TsKvProto',
  full_name='messages.TsKvProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ts', full_name='messages.TsKvProto.ts', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kv', full_name='messages.TsKvProto.kv', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1960,
  serialized_end=2020,
)


_TSKVLISTPROTO = _descriptor.Descriptor(
  name='TsKvListProto',
  full_name='messages.TsKvListProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ts', full_name='messages.TsKvListProto.ts', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='kv', full_name='messages.TsKvListProto.kv', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2022,
  serialized_end=2086,
)


_POSTTELEMETRYMSG = _descriptor.Descriptor(
  name='PostTelemetryMsg',
  full_name='messages.PostTelemetryMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tsKvList', full_name='messages.PostTelemetryMsg.tsKvList', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2088,
  serialized_end=2149,
)


_POSTATTRIBUTEMSG = _descriptor.Descriptor(
  name='PostAttributeMsg',
  full_name='messages.PostAttributeMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='kv', full_name='messages.PostAttributeMsg.kv', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2151,
  serialized_end=2206,
)


_ATTRIBUTEUPDATENOTIFICATIONMSG = _descriptor.Descriptor(
  name='AttributeUpdateNotificationMsg',
  full_name='messages.AttributeUpdateNotificationMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sharedUpdated', full_name='messages.AttributeUpdateNotificationMsg.sharedUpdated', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sharedDeleted', full_name='messages.AttributeUpdateNotificationMsg.sharedDeleted', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2208,
  serialized_end=2307,
)


_TODEVICERPCREQUESTMSG = _descriptor.Descriptor(
  name='ToDeviceRpcRequestMsg',
  full_name='messages.ToDeviceRpcRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToDeviceRpcRequestMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='methodName', full_name='messages.ToDeviceRpcRequestMsg.methodName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='messages.ToDeviceRpcRequestMsg.params', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2309,
  serialized_end=2387,
)


_TODEVICERPCRESPONSEMSG = _descriptor.Descriptor(
  name='ToDeviceRpcResponseMsg',
  full_name='messages.ToDeviceRpcResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToDeviceRpcResponseMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='messages.ToDeviceRpcResponseMsg.payload', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='messages.ToDeviceRpcResponseMsg.error', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2389,
  serialized_end=2464,
)


_TOSERVERRPCREQUESTMSG = _descriptor.Descriptor(
  name='ToServerRpcRequestMsg',
  full_name='messages.ToServerRpcRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToServerRpcRequestMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='methodName', full_name='messages.ToServerRpcRequestMsg.methodName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='messages.ToServerRpcRequestMsg.params', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2466,
  serialized_end=2544,
)


_TOSERVERRPCRESPONSEMSG = _descriptor.Descriptor(
  name='ToServerRpcResponseMsg',
  full_name='messages.ToServerRpcResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.ToServerRpcResponseMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='messages.ToServerRpcResponseMsg.payload', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='messages.ToServerRpcResponseMsg.error', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2546,
  serialized_end=2621,
)


_CLAIMDEVICE = _descriptor.Descriptor(
  name='ClaimDevice',
  full_name='messages.ClaimDevice',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='secretKey', full_name='messages.ClaimDevice.secretKey', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='durationMs', full_name='messages.ClaimDevice.durationMs', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2623,
  serialized_end=2675,
)


_ATTRIBUTESREQUEST = _descriptor.Descriptor(
  name='AttributesRequest',
  full_name='messages.AttributesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='clientKeys', full_name='messages.AttributesRequest.clientKeys', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sharedKeys', full_name='messages.AttributesRequest.sharedKeys', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2677,
  serialized_end=2736,
)


_RPCREQUEST = _descriptor.Descriptor(
  name='RpcRequest',
  full_name='messages.RpcRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='method', full_name='messages.RpcRequest.method', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='params', full_name='messages.RpcRequest.params', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2738,
  serialized_end=2782,
)


_DISCONNECTMSG = _descriptor.Descriptor(
  name='DisconnectMsg',
  full_name='messages.DisconnectMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.DisconnectMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2784,
  serialized_end=2819,
)


_CONNECTMSG = _descriptor.Descriptor(
  name='ConnectMsg',
  full_name='messages.ConnectMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.ConnectMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceType', full_name='messages.ConnectMsg.deviceType', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2821,
  serialized_end=2873,
)


_TELEMETRYMSG = _descriptor.Descriptor(
  name='TelemetryMsg',
  full_name='messages.TelemetryMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.TelemetryMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.TelemetryMsg.msg', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2875,
  serialized_end=2950,
)


_ATTRIBUTESMSG = _descriptor.Descriptor(
  name='AttributesMsg',
  full_name='messages.AttributesMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.AttributesMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.AttributesMsg.msg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2952,
  serialized_end=3028,
)


_CLAIMDEVICEMSG = _descriptor.Descriptor(
  name='ClaimDeviceMsg',
  full_name='messages.ClaimDeviceMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.ClaimDeviceMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='claimRequest', full_name='messages.ClaimDeviceMsg.claimRequest', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3030,
  serialized_end=3111,
)


_GATEWAYTELEMETRYMSG = _descriptor.Descriptor(
  name='GatewayTelemetryMsg',
  full_name='messages.GatewayTelemetryMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.GatewayTelemetryMsg.msg', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3113,
  serialized_end=3171,
)


_GATEWAYCLAIMMSG = _descriptor.Descriptor(
  name='GatewayClaimMsg',
  full_name='messages.GatewayClaimMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.GatewayClaimMsg.msg', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3173,
  serialized_end=3229,
)


_GATEWAYATTRIBUTESMSG = _descriptor.Descriptor(
  name='GatewayAttributesMsg',
  full_name='messages.GatewayAttributesMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='msg', full_name='messages.GatewayAttributesMsg.msg', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3231,
  serialized_end=3291,
)


_GATEWAYRPCRESPONSEMSG = _descriptor.Descriptor(
  name='GatewayRpcResponseMsg',
  full_name='messages.GatewayRpcResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayRpcResponseMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='messages.GatewayRpcResponseMsg.id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='messages.GatewayRpcResponseMsg.data', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3293,
  serialized_end=3362,
)


_GATEWAYATTRIBUTERESPONSEMSG = _descriptor.Descriptor(
  name='GatewayAttributeResponseMsg',
  full_name='messages.GatewayAttributeResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayAttributeResponseMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='responseMsg', full_name='messages.GatewayAttributeResponseMsg.responseMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3364,
  serialized_end=3474,
)


_GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG = _descriptor.Descriptor(
  name='GatewayAttributeUpdateNotificationMsg',
  full_name='messages.GatewayAttributeUpdateNotificationMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayAttributeUpdateNotificationMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='notificationMsg', full_name='messages.GatewayAttributeUpdateNotificationMsg.notificationMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3476,
  serialized_end=3602,
)


_GATEWAYDEVICERPCREQUESTMSG = _descriptor.Descriptor(
  name='GatewayDeviceRpcRequestMsg',
  full_name='messages.GatewayDeviceRpcRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayDeviceRpcRequestMsg.deviceName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rpcRequestMsg', full_name='messages.GatewayDeviceRpcRequestMsg.rpcRequestMsg', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3604,
  serialized_end=3708,
)


_GATEWAYATTRIBUTESREQUESTMSG = _descriptor.Descriptor(
  name='GatewayAttributesRequestMsg',
  full_name='messages.GatewayAttributesRequestMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='messages.GatewayAttributesRequestMsg.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='messages.GatewayAttributesRequestMsg.deviceName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client', full_name='messages.GatewayAttributesRequestMsg.client', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='keys', full_name='messages.GatewayAttributesRequestMsg.keys', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3710,
  serialized_end=3801,
)


_GATEWAYATTRIBUTESRESPONSEMSG = _descriptor.Descriptor(
  name='GatewayAttributesResponseMsg',
  full_name='messages.GatewayAttributesResponseMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='requestId', full_name='messages.GatewayAttributesResponseMsg.requestId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clientAttributeList', full_name='messages.GatewayAttributesResponseMsg.clientAttributeList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sharedAttributeList', full_name='messages.GatewayAttributesResponseMsg.sharedAttributeList', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='messages.GatewayAttributesResponseMsg.error', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3804,
  serialized_end=3976,
)

_RESPONSE.fields_by_name['status'].enum_type = _RESPONSESTATUS
_RESPONSE.fields_by_name['serviceMessage'].message_type = _FROMSERVICEMESSAGE
_RESPONSE.fields_by_name['connectorMessage'].message_type = _FROMCONNECTORMESSAGE
_FROMCONNECTORMESSAGE.fields_by_name['response'].message_type = _RESPONSE
_FROMCONNECTORMESSAGE.fields_by_name['gatewayTelemetryMsg'].message_type = _GATEWAYTELEMETRYMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayAttributesMsg'].message_type = _GATEWAYATTRIBUTESMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayClaimMsg'].message_type = _GATEWAYCLAIMMSG
_FROMCONNECTORMESSAGE.fields_by_name['registerConnectorMsg'].message_type = _REGISTERCONNECTORMSG
_FROMCONNECTORMESSAGE.fields_by_name['unregisterConnectorMsg'].message_type = _UNREGISTERCONNECTORMSG
_FROMCONNECTORMESSAGE.fields_by_name['connectMsg'].message_type = _CONNECTMSG
_FROMCONNECTORMESSAGE.fields_by_name['disconnectMsg'].message_type = _DISCONNECTMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayRpcResponseMsg'].message_type = _GATEWAYRPCRESPONSEMSG
_FROMCONNECTORMESSAGE.fields_by_name['gatewayAttributeRequestMsg'].message_type = _GATEWAYATTRIBUTESREQUESTMSG
_FROMCONNECTORMESSAGE.fields_by_name['connectorGetConnectedDevicesMsg'].message_type = _CONNECTORGETCONNECTEDDEVICESMSG
_FROMSERVICEMESSAGE.fields_by_name['response'].message_type = _RESPONSE
_FROMSERVICEMESSAGE.fields_by_name['connectorConfigurationMsg'].message_type = _CONNECTORCONFIGURATIONMSG
_FROMSERVICEMESSAGE.fields_by_name['gatewayAttributeUpdateNotificationMsg'].message_type = _GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG
_FROMSERVICEMESSAGE.fields_by_name['gatewayAttributeResponseMsg'].message_type = _GATEWAYATTRIBUTERESPONSEMSG
_FROMSERVICEMESSAGE.fields_by_name['gatewayDeviceRpcRequestMsg'].message_type = _GATEWAYDEVICERPCREQUESTMSG
_FROMSERVICEMESSAGE.fields_by_name['unregisterConnectorMsg'].message_type = _UNREGISTERCONNECTORMSG
_FROMSERVICEMESSAGE.fields_by_name['connectorGetConnectedDevicesResponseMsg'].message_type = _CONNECTORGETCONNECTEDDEVICESRESPONSEMSG
_CONNECTORGETCONNECTEDDEVICESRESPONSEMSG.fields_by_name['connectorDevices'].message_type = _CONNECTORDEVICEINFO
_KEYVALUEPROTO.fields_by_name['type'].enum_type = _KEYVALUETYPE
_TSKVPROTO.fields_by_name['kv'].message_type = _KEYVALUEPROTO
_TSKVLISTPROTO.fields_by_name['kv'].message_type = _KEYVALUEPROTO
_POSTTELEMETRYMSG.fields_by_name['tsKvList'].message_type = _TSKVLISTPROTO
_POSTATTRIBUTEMSG.fields_by_name['kv'].message_type = _KEYVALUEPROTO
_ATTRIBUTEUPDATENOTIFICATIONMSG.fields_by_name['sharedUpdated'].message_type = _TSKVPROTO
_TELEMETRYMSG.fields_by_name['msg'].message_type = _POSTTELEMETRYMSG
_ATTRIBUTESMSG.fields_by_name['msg'].message_type = _POSTATTRIBUTEMSG
_CLAIMDEVICEMSG.fields_by_name['claimRequest'].message_type = _CLAIMDEVICE
_GATEWAYTELEMETRYMSG.fields_by_name['msg'].message_type = _TELEMETRYMSG
_GATEWAYCLAIMMSG.fields_by_name['msg'].message_type = _CLAIMDEVICEMSG
_GATEWAYATTRIBUTESMSG.fields_by_name['msg'].message_type = _ATTRIBUTESMSG
_GATEWAYATTRIBUTERESPONSEMSG.fields_by_name['responseMsg'].message_type = _GATEWAYATTRIBUTESRESPONSEMSG
_GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG.fields_by_name['notificationMsg'].message_type = _ATTRIBUTEUPDATENOTIFICATIONMSG
_GATEWAYDEVICERPCREQUESTMSG.fields_by_name['rpcRequestMsg'].message_type = _TODEVICERPCREQUESTMSG
_GATEWAYATTRIBUTESRESPONSEMSG.fields_by_name['clientAttributeList'].message_type = _KEYVALUEPROTO
_GATEWAYATTRIBUTESRESPONSEMSG.fields_by_name['sharedAttributeList'].message_type = _KEYVALUEPROTO
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['FromConnectorMessage'] = _FROMCONNECTORMESSAGE
DESCRIPTOR.message_types_by_name['FromServiceMessage'] = _FROMSERVICEMESSAGE
DESCRIPTOR.message_types_by_name['RegisterConnectorMsg'] = _REGISTERCONNECTORMSG
DESCRIPTOR.message_types_by_name['UnregisterConnectorMsg'] = _UNREGISTERCONNECTORMSG
DESCRIPTOR.message_types_by_name['ConnectorConfigurationMsg'] = _CONNECTORCONFIGURATIONMSG
DESCRIPTOR.message_types_by_name['ConnectorGetConnectedDevicesMsg'] = _CONNECTORGETCONNECTEDDEVICESMSG
DESCRIPTOR.message_types_by_name['ConnectorGetConnectedDevicesResponseMsg'] = _CONNECTORGETCONNECTEDDEVICESRESPONSEMSG
DESCRIPTOR.message_types_by_name['ConnectorDeviceInfo'] = _CONNECTORDEVICEINFO
DESCRIPTOR.message_types_by_name['KeyValueProto'] = _KEYVALUEPROTO
DESCRIPTOR.message_types_by_name['TsKvProto'] = _TSKVPROTO
DESCRIPTOR.message_types_by_name['TsKvListProto'] = _TSKVLISTPROTO
DESCRIPTOR.message_types_by_name['PostTelemetryMsg'] = _POSTTELEMETRYMSG
DESCRIPTOR.message_types_by_name['PostAttributeMsg'] = _POSTATTRIBUTEMSG
DESCRIPTOR.message_types_by_name['AttributeUpdateNotificationMsg'] = _ATTRIBUTEUPDATENOTIFICATIONMSG
DESCRIPTOR.message_types_by_name['ToDeviceRpcRequestMsg'] = _TODEVICERPCREQUESTMSG
DESCRIPTOR.message_types_by_name['ToDeviceRpcResponseMsg'] = _TODEVICERPCRESPONSEMSG
DESCRIPTOR.message_types_by_name['ToServerRpcRequestMsg'] = _TOSERVERRPCREQUESTMSG
DESCRIPTOR.message_types_by_name['ToServerRpcResponseMsg'] = _TOSERVERRPCRESPONSEMSG
DESCRIPTOR.message_types_by_name['ClaimDevice'] = _CLAIMDEVICE
DESCRIPTOR.message_types_by_name['AttributesRequest'] = _ATTRIBUTESREQUEST
DESCRIPTOR.message_types_by_name['RpcRequest'] = _RPCREQUEST
DESCRIPTOR.message_types_by_name['DisconnectMsg'] = _DISCONNECTMSG
DESCRIPTOR.message_types_by_name['ConnectMsg'] = _CONNECTMSG
DESCRIPTOR.message_types_by_name['TelemetryMsg'] = _TELEMETRYMSG
DESCRIPTOR.message_types_by_name['AttributesMsg'] = _ATTRIBUTESMSG
DESCRIPTOR.message_types_by_name['ClaimDeviceMsg'] = _CLAIMDEVICEMSG
DESCRIPTOR.message_types_by_name['GatewayTelemetryMsg'] = _GATEWAYTELEMETRYMSG
DESCRIPTOR.message_types_by_name['GatewayClaimMsg'] = _GATEWAYCLAIMMSG
DESCRIPTOR.message_types_by_name['GatewayAttributesMsg'] = _GATEWAYATTRIBUTESMSG
DESCRIPTOR.message_types_by_name['GatewayRpcResponseMsg'] = _GATEWAYRPCRESPONSEMSG
DESCRIPTOR.message_types_by_name['GatewayAttributeResponseMsg'] = _GATEWAYATTRIBUTERESPONSEMSG
DESCRIPTOR.message_types_by_name['GatewayAttributeUpdateNotificationMsg'] = _GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG
DESCRIPTOR.message_types_by_name['GatewayDeviceRpcRequestMsg'] = _GATEWAYDEVICERPCREQUESTMSG
DESCRIPTOR.message_types_by_name['GatewayAttributesRequestMsg'] = _GATEWAYATTRIBUTESREQUESTMSG
DESCRIPTOR.message_types_by_name['GatewayAttributesResponseMsg'] = _GATEWAYATTRIBUTESRESPONSEMSG
DESCRIPTOR.enum_types_by_name['ResponseStatus'] = _RESPONSESTATUS
DESCRIPTOR.enum_types_by_name['KeyValueType'] = _KEYVALUETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.Response)
  })
_sym_db.RegisterMessage(Response)

FromConnectorMessage = _reflection.GeneratedProtocolMessageType('FromConnectorMessage', (_message.Message,), {
  'DESCRIPTOR' : _FROMCONNECTORMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.FromConnectorMessage)
  })
_sym_db.RegisterMessage(FromConnectorMessage)

FromServiceMessage = _reflection.GeneratedProtocolMessageType('FromServiceMessage', (_message.Message,), {
  'DESCRIPTOR' : _FROMSERVICEMESSAGE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.FromServiceMessage)
  })
_sym_db.RegisterMessage(FromServiceMessage)

RegisterConnectorMsg = _reflection.GeneratedProtocolMessageType('RegisterConnectorMsg', (_message.Message,), {
  'DESCRIPTOR' : _REGISTERCONNECTORMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.RegisterConnectorMsg)
  })
_sym_db.RegisterMessage(RegisterConnectorMsg)

UnregisterConnectorMsg = _reflection.GeneratedProtocolMessageType('UnregisterConnectorMsg', (_message.Message,), {
  'DESCRIPTOR' : _UNREGISTERCONNECTORMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.UnregisterConnectorMsg)
  })
_sym_db.RegisterMessage(UnregisterConnectorMsg)

ConnectorConfigurationMsg = _reflection.GeneratedProtocolMessageType('ConnectorConfigurationMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORCONFIGURATIONMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorConfigurationMsg)
  })
_sym_db.RegisterMessage(ConnectorConfigurationMsg)

ConnectorGetConnectedDevicesMsg = _reflection.GeneratedProtocolMessageType('ConnectorGetConnectedDevicesMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORGETCONNECTEDDEVICESMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorGetConnectedDevicesMsg)
  })
_sym_db.RegisterMessage(ConnectorGetConnectedDevicesMsg)

ConnectorGetConnectedDevicesResponseMsg = _reflection.GeneratedProtocolMessageType('ConnectorGetConnectedDevicesResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORGETCONNECTEDDEVICESRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorGetConnectedDevicesResponseMsg)
  })
_sym_db.RegisterMessage(ConnectorGetConnectedDevicesResponseMsg)

ConnectorDeviceInfo = _reflection.GeneratedProtocolMessageType('ConnectorDeviceInfo', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTORDEVICEINFO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectorDeviceInfo)
  })
_sym_db.RegisterMessage(ConnectorDeviceInfo)

KeyValueProto = _reflection.GeneratedProtocolMessageType('KeyValueProto', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUEPROTO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.KeyValueProto)
  })
_sym_db.RegisterMessage(KeyValueProto)

TsKvProto = _reflection.GeneratedProtocolMessageType('TsKvProto', (_message.Message,), {
  'DESCRIPTOR' : _TSKVPROTO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.TsKvProto)
  })
_sym_db.RegisterMessage(TsKvProto)

TsKvListProto = _reflection.GeneratedProtocolMessageType('TsKvListProto', (_message.Message,), {
  'DESCRIPTOR' : _TSKVLISTPROTO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.TsKvListProto)
  })
_sym_db.RegisterMessage(TsKvListProto)

PostTelemetryMsg = _reflection.GeneratedProtocolMessageType('PostTelemetryMsg', (_message.Message,), {
  'DESCRIPTOR' : _POSTTELEMETRYMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.PostTelemetryMsg)
  })
_sym_db.RegisterMessage(PostTelemetryMsg)

PostAttributeMsg = _reflection.GeneratedProtocolMessageType('PostAttributeMsg', (_message.Message,), {
  'DESCRIPTOR' : _POSTATTRIBUTEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.PostAttributeMsg)
  })
_sym_db.RegisterMessage(PostAttributeMsg)

AttributeUpdateNotificationMsg = _reflection.GeneratedProtocolMessageType('AttributeUpdateNotificationMsg', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTEUPDATENOTIFICATIONMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.AttributeUpdateNotificationMsg)
  })
_sym_db.RegisterMessage(AttributeUpdateNotificationMsg)

ToDeviceRpcRequestMsg = _reflection.GeneratedProtocolMessageType('ToDeviceRpcRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _TODEVICERPCREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToDeviceRpcRequestMsg)
  })
_sym_db.RegisterMessage(ToDeviceRpcRequestMsg)

ToDeviceRpcResponseMsg = _reflection.GeneratedProtocolMessageType('ToDeviceRpcResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _TODEVICERPCRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToDeviceRpcResponseMsg)
  })
_sym_db.RegisterMessage(ToDeviceRpcResponseMsg)

ToServerRpcRequestMsg = _reflection.GeneratedProtocolMessageType('ToServerRpcRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _TOSERVERRPCREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToServerRpcRequestMsg)
  })
_sym_db.RegisterMessage(ToServerRpcRequestMsg)

ToServerRpcResponseMsg = _reflection.GeneratedProtocolMessageType('ToServerRpcResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _TOSERVERRPCRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ToServerRpcResponseMsg)
  })
_sym_db.RegisterMessage(ToServerRpcResponseMsg)

ClaimDevice = _reflection.GeneratedProtocolMessageType('ClaimDevice', (_message.Message,), {
  'DESCRIPTOR' : _CLAIMDEVICE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ClaimDevice)
  })
_sym_db.RegisterMessage(ClaimDevice)

AttributesRequest = _reflection.GeneratedProtocolMessageType('AttributesRequest', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTESREQUEST,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.AttributesRequest)
  })
_sym_db.RegisterMessage(AttributesRequest)

RpcRequest = _reflection.GeneratedProtocolMessageType('RpcRequest', (_message.Message,), {
  'DESCRIPTOR' : _RPCREQUEST,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.RpcRequest)
  })
_sym_db.RegisterMessage(RpcRequest)

DisconnectMsg = _reflection.GeneratedProtocolMessageType('DisconnectMsg', (_message.Message,), {
  'DESCRIPTOR' : _DISCONNECTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.DisconnectMsg)
  })
_sym_db.RegisterMessage(DisconnectMsg)

ConnectMsg = _reflection.GeneratedProtocolMessageType('ConnectMsg', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ConnectMsg)
  })
_sym_db.RegisterMessage(ConnectMsg)

TelemetryMsg = _reflection.GeneratedProtocolMessageType('TelemetryMsg', (_message.Message,), {
  'DESCRIPTOR' : _TELEMETRYMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.TelemetryMsg)
  })
_sym_db.RegisterMessage(TelemetryMsg)

AttributesMsg = _reflection.GeneratedProtocolMessageType('AttributesMsg', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTESMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.AttributesMsg)
  })
_sym_db.RegisterMessage(AttributesMsg)

ClaimDeviceMsg = _reflection.GeneratedProtocolMessageType('ClaimDeviceMsg', (_message.Message,), {
  'DESCRIPTOR' : _CLAIMDEVICEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.ClaimDeviceMsg)
  })
_sym_db.RegisterMessage(ClaimDeviceMsg)

GatewayTelemetryMsg = _reflection.GeneratedProtocolMessageType('GatewayTelemetryMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYTELEMETRYMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayTelemetryMsg)
  })
_sym_db.RegisterMessage(GatewayTelemetryMsg)

GatewayClaimMsg = _reflection.GeneratedProtocolMessageType('GatewayClaimMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYCLAIMMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayClaimMsg)
  })
_sym_db.RegisterMessage(GatewayClaimMsg)

GatewayAttributesMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributesMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTESMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributesMsg)
  })
_sym_db.RegisterMessage(GatewayAttributesMsg)

GatewayRpcResponseMsg = _reflection.GeneratedProtocolMessageType('GatewayRpcResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYRPCRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayRpcResponseMsg)
  })
_sym_db.RegisterMessage(GatewayRpcResponseMsg)

GatewayAttributeResponseMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributeResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTERESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributeResponseMsg)
  })
_sym_db.RegisterMessage(GatewayAttributeResponseMsg)

GatewayAttributeUpdateNotificationMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributeUpdateNotificationMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTEUPDATENOTIFICATIONMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributeUpdateNotificationMsg)
  })
_sym_db.RegisterMessage(GatewayAttributeUpdateNotificationMsg)

GatewayDeviceRpcRequestMsg = _reflection.GeneratedProtocolMessageType('GatewayDeviceRpcRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYDEVICERPCREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayDeviceRpcRequestMsg)
  })
_sym_db.RegisterMessage(GatewayDeviceRpcRequestMsg)

GatewayAttributesRequestMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributesRequestMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTESREQUESTMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributesRequestMsg)
  })
_sym_db.RegisterMessage(GatewayAttributesRequestMsg)

GatewayAttributesResponseMsg = _reflection.GeneratedProtocolMessageType('GatewayAttributesResponseMsg', (_message.Message,), {
  'DESCRIPTOR' : _GATEWAYATTRIBUTESRESPONSEMSG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:messages.GatewayAttributesResponseMsg)
  })
_sym_db.RegisterMessage(GatewayAttributesResponseMsg)



_TBGATEWAYPROTOSERVICE = _descriptor.ServiceDescriptor(
  name='TBGatewayProtoService',
  full_name='messages.TBGatewayProtoService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=4133,
  serialized_end=4228,
  methods=[
  _descriptor.MethodDescriptor(
    name='stream',
    full_name='messages.TBGatewayProtoService.stream',
    index=0,
    containing_service=None,
    input_type=_FROMCONNECTORMESSAGE,
    output_type=_FROMSERVICEMESSAGE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TBGATEWAYPROTOSERVICE)

DESCRIPTOR.services_by_name['TBGatewayProtoService'] = _TBGATEWAYPROTOSERVICE

# @@protoc_insertion_point(module_scope)
